#===========================================================================================================================#
#===========================================================================================================================#
# MASTER SCRIPT Unravelling the Complex Role of Natural Resources 2019                                                                                                                    #
#===========================================================================================================================#
#===========================================================================================================================#

# Description: this scripts aims to investigate the relevance of including natural resource variables for 
# conflict risk predictions by comparing the predictive performance of less vs. more complex modelling approaches: 
# the common logistic regression, a one-hidden-layer neural network, a two-hidden-layers neural network and a 
# random forest model. 

# This script loads a prepared Rdataframe from the processed data folder. The variables that the EC JRC has intellectual
# property rights are replaced with NAs. That means only the new natural resource variables are visible in it for now.
# to understand the missing JRC variables check https://ec.europa.eu/jrc/en/peace-and-stability/projects/gcri/technical-documentation
#    - The originally downloaded natural resource data can be found in the folder ORG_DATA
#    - The scripts to process them (read, imputation, normalisation etc.) can be found in the folder 'scripts to process data'
#    - Each natural resource variable in it processed form can be found in the folder PROCESSED_DATA
#    - All predictor variables are put together into the Rdataframa 'incomplete.data.Rdata', which you can also find under the 
#      PROCESSES_DATA folder

# This script organizes all variables into the four variable sets (see paper), then calls the four different modelling 
# approaches (linear regression, neural network and random forest) under a 10-fold cross-validation loop. Lastly, it 
# performs a wide range of predictive model performance calculations to compare the developed models and analyses the
# predictor importance and interactions of variables in the random forest models.

# Make sure to fill in all 'XXX' according to your specifics: e.g. working directory in line 42.
# Questions or comments always welcome: marie.schellens@natgeo.su.se or marieschellens@msn.com


#===========================================================================================================================#

#### Set-up                                                                                                 ####

#===========================================================================================================================#

# if you don't want to rerun the modelling procedure, you can just load the models and workspace from line 264 
# and work further from there

#### prepare workspace ####
#--------------------------

setwd("XXX")

# clean up space
rm(list = ls(all = TRUE)) # alles opruimen

# settings to choose and constants to set
data_to_load <- "PROCESSED_DATA/incomplete.data.Rdata"
k <- 10 # number of folds for the k-fold crossvalidation
learning_threshold <- 0.3 #1.2
learning_threshold_2 <- 0.05 # default is 0.01
ntree <- 500; # Number of trees to grow. This should not be set to too small a number, to ensure that every input row gets predicted at least a few times.
modelling_approaches <- c ("Logistic regression", "Random forest", "Neural network 1", "Neural network 2")
nr_of_models <- length(modelling_approaches); # nodig om de lege matrices te maken die ingevuld zullen worden
predictorVars_set_names <- c("Original set", "Socio-economic set", "Natural resource set","Combined set")
nr_of_variable_sets <- length(predictorVars_set_names)

# download, install and call required packages (both works with function 'library' and 'require')
#install.packages("pROC") # Run this if you didn't download them yet 
library(pROC)     # For Receiver Operating Characteristic curve and Area Under Curve metric
#install.packages("neuralnet")
require(neuralnet)
#install.packages("nnet") # need a small function of this package to change factor variable into a matrix with occurence per class
require(nnet) 
#library(neuralnet) # this one or the previous line, do the same, work as well
#install.packages("randomForest")
library(randomForest)
#install.packages("randomForestExplainer") 
library(randomForestExplainer)
#install.packages("SDMTools") # package for creating confusion matrix, Caret package seems to be a decent one too for predictive modelling
library(SDMTools)

# setting the seed
set.seed(123)


#### Load and prep data ####
#---------------------------

load(data_to_load)
load("data/COUNTRY_SIZE.Rdata") # to plot residuals vs country sizes in the end

incomplete.data <- incomplete.data[with(incomplete.data, order(ISO, -YEAR)), ] # put them in right order

# Create Boolean for existence of violent conflict over a threshold of 25 deaths per year
incomplete.data$conflict_boolean <- #XXX # created based on GCRI's conflict intensity data not given here
depVar <- "conflict_boolean"
incomplete.data$conflict_boolean <- factor(incomplete.data$conflict_boolean, levels = c("0","1"), labels = c("peace","conflict")) 

incomplete.data$ETHNIC<-pmax(incomplete.data$ETHNIC_NP,incomplete.data$ETHNIC_SN) # create an ethnicity variable independent of national or subnational conflict

# Check the data we want to predict
hist(incomplete.data$conflict_boolean)
#hist(incomplete.data$conflict_boolean_NP)
#hist(incomplete.data$conflict_boolean_SN)
#hist(incomplete.data$Intensity_Y4_NP)
#hist(incomplete.data$Intensity_Y4_SN)

# prepare the sets of predictor variables
allVars<-colnames(incomplete.data)
remove <- c ("ISO", "YEAR", "COUNTRY", "Intensity_Y_NP","Intensity_Y_SN","Intensity_Y4_NP", "Intensity_Y4_SN",
             "VC_Y_NP", "VC_Y_SN", "conflict_boolean_NP", "conflict_boolean_SN","HVC_Y_NP", "HVC_Y_SN", "conflict_boolean", "REGION")
predictorVars<-allVars[! allVars %in% remove] # all variables

predictorVarsOrigEnv <- c ("FOOD","FUEL_EXP","POP","STRUCT","WATER") # environmental/geographical GCRI variables
predictorVarsNewEnv <- c ("ACCESSIBILITY","AIR_POLLUTION","ARABLE_LAND","BIODIVERSITY_PROT","FOOD_PROD",
                          "FOREST_AREA","NAT_DISASTER","ORE_EXPORT","POPULATION_DENSITY","RENEWABLE_ELEC",
                          "RESOURCE_RENT","SOIL_DEGR","WATER_ACCESS","WATER_RES_PER_CAP","WATER_WITHDR",
                          "FOOD","FUEL_EXP") # Only, and new, Natural resource set
predictorVarsNewEnvFish <- c ("FISH", predictorVarsNewEnv) # Only, and new, Natural resource set (with fish collapse)

predictorVarsOrig <- predictorVars[! predictorVars %in% predictorVarsNewEnvFish] # Original set
  predictorVarsOrig <- predictorVarsOrig[! predictorVarsOrig %in% c("ETHNIC_NP","ETHNIC_SN")]
  predictorVarsOrig <- c (predictorVarsOrig,"FOOD","FUEL_EXP")
predictorVarsOrigSocioEcon <- predictorVarsOrig [! predictorVarsOrig %in% predictorVarsOrigEnv] # Socio-economic GCRI variables (without Natural resource set)

predictorVarsOrigNewEnv <- c (predictorVarsOrigSocioEcon,predictorVarsNewEnv) # Original set and extra, new Natural resource set
predictorVarsOrigNewEnvFish <- c ("FISH",predictorVarsOrigNewEnv) # Original set and extra, new Natural resource set (with fish collapse)


#===========================================================================================================================#

#### Building the model                                                                                                 ####

#===========================================================================================================================#

# create empty lists to save the models in
# ----------------------------------------

fold_1 <- list () ## verander de i van de naam van de loop over 10 folds om alles volledig automtisch te maken
fold_2 <- list ()
fold_3 <- list ()
fold_4 <- list ()
fold_5 <- list ()
fold_6 <- list ()
fold_7 <- list ()
fold_8 <- list ()
fold_9 <- list ()
fold_10 <- list ()

Original_GCRI_var_models <- list (fold_1, fold_2, fold_3, fold_4, fold_5, fold_6, fold_7, fold_8, fold_9, fold_10)
Socio_econom_var_models <- list (fold_1, fold_2, fold_3, fold_4, fold_5, fold_6, fold_7, fold_8, fold_9, fold_10)
Env_var_models <- list (fold_1, fold_2, fold_3, fold_4, fold_5, fold_6, fold_7, fold_8, fold_9, fold_10)
Original_GCRI_env_var_models <- list (fold_1, fold_2, fold_3, fold_4, fold_5, fold_6, fold_7, fold_8, fold_9, fold_10)

logit.models <- list(Original_GCRI_var_models,Socio_econom_var_models,Env_var_models,Original_GCRI_env_var_models)
randomforest.models <- list(Original_GCRI_var_models,Socio_econom_var_models,Env_var_models,Original_GCRI_env_var_models)
neuralnet1.models <- list(Original_GCRI_var_models,Socio_econom_var_models,Env_var_models,Original_GCRI_env_var_models)
neuralnet2.models <- list(Original_GCRI_var_models,Socio_econom_var_models,Env_var_models,Original_GCRI_env_var_models)

list_all_models <- list(logit.models,randomforest.models,neuralnet1.models,neuralnet2.models)

rm(logit.models,randomforest.models,neuralnet1.models,neuralnet2.models,Original_GCRI_var_models,Socio_econom_var_models,Env_var_models,Original_GCRI_env_var_models)

train.sets <- list()
test.sets <- list()

# split into training and validation set for cross-validation (k-fold)
# -----------------------------------------------------------

# Subset units with data on dependent variable: The dataset is subset to only the units that have data on the dummy variable for conflict 4 years ahead in time.
data <- subset(incomplete.data, YEAR<=(max(incomplete.data$YEAR)-4))

# and omit all the rows with NAs, but only for the variabels that we will use in this particular analysis
data <- subset(data,select = c("ISO","YEAR","COUNTRY","REGION",depVar,"Intensity_Y_NP","Intensity_Y_SN","Intensity_Y4_NP","Intensity_Y4_SN",predictorVarsOrig,predictorVarsNewEnv))
#data <- na.omit(data) # zou normaal niets moeten veranderen als de imputatie volledig was, doet toch nog rijen weg omdat, voor sommige landen geen enkel datapunt gevonden is voor soil degradation

#Randomly shuffle the data (by taking a random sample as big as the original dataset)
data<-data[sample(nrow(data)),]

#Create k equally size folds
folds <- cut(seq(1,nrow(data)),breaks=k,labels=FALSE) 
# seq geeft gewoon 1 lange rij met elk nummer van het aantal observaties
# cut verdeeld deze rij over k folds, elke observatie krijgt een integer/een fold aangewezen

#Perform k fold cross (training and) validation, 
for(i in 1:k){
  #Segment your data by fold using the which() function and save the training and testing sets as data frames
  print(paste("k-fold cross validation iteration number",i))
  testIndexes <- which(folds==i,arr.ind=TRUE) # this line indexes as TRUE if the observation is part of the current test fold
  test <- data[testIndexes, ]
    test.sets[[i]] <- test
  train <- data[-testIndexes, ]
    train.sets[[i]] <- train
  #Use the test and train data partitions however you desire...
  #(loop will continues all the way until performance measure of the model)

    
#### Start loop over the 4 different type of models here
# ------------------------------------------------------
# (multiple linear regression, (2) neural net and random forest)
# configuration of the model only after construction of training and test data sets

for(l in 1:nr_of_models){
  print(paste("    model approach calculating now is",modelling_approaches[l]))  

  
#### loop over all predictor variable sets!
#------------------------------------------
  
predictorVars_sets <- list(predictorVarsOrig,predictorVarsOrigSocioEcon,predictorVarsNewEnv,predictorVarsOrigNewEnv)
  
for(j in 1:nr_of_variable_sets){ #loop loopt tot ergens in het einde van de performance berekeningem
  ChosenPredVars <- predictorVars_sets[[j]] # dubbele haken om de vector in de list te krijgen ipv een deeltje van de list gewoon
  print(paste("        Chosen Pred Vars Set is",predictorVars_set_names[j])) # to follow where we are in the calculations
    

#### configuration of models: multiple linear regression, neural net and random forest
#-------------------------------------------------------------------------------------

predictorVars_for_form<- paste(ChosenPredVars,collapse = "+")
form=as.formula(paste(paste(depVar,"~",sep=""),predictorVars_for_form)) # schrijft de vergelijking uit

if (l == 1) {
  # logistic regression
  LogisticModel <- glm(formula=form, data=train, family=binomial)
  
  list_all_models[[l]][[j]][[i]]<-LogisticModel
} else if (l == 2) {
  # random forest
  RandomForestModel <- randomForest(form,train, ntree = ntree,localImp=TRUE,na.action=na.omit,replace=TRUE,sampsize = ceiling(.632*nrow(train))) # calculation of local importance needed for visualisations

  list_all_models[[l]][[j]][[i]]<-RandomForestModel
} else if (l == 3) {
  # neural network 1 layer
  train_for_NN <- cbind(train, class.ind(train$conflict_boolean)) # creates a membership matrix for the two levels of the factor variable conflict_boolean zodat we een classificatie neuralnet kunnen maken
  train_for_NN <- na.omit(train_for_NN)
  form <- as.formula(paste("peace + conflict ~", predictorVars_for_form))
  n_hidden_nodes <- round(sqrt(length(predictorVars)*length(levels(train$conflict_boolean)))) # Masters' geometric pyramid rule (of thumb)
  
  NeuralModel1 <- neuralnet(formula=form, data=train_for_NN, rep=1, hidden=n_hidden_nodes, act.fct = "logistic", linear.output=FALSE, threshold = learning_threshold, lifesign = 'full') 

  # the argument linear.output to FALSE in order to tell the model that I want to apply the activation ... 
  # ... function act.fct and that I am not doing a regression task. Then I set the activation function to ...
  # ... logistic (which by the way is the default option) in order to apply the logistic function. 
  # hidden is a vector of integers specifying the number of hidden neurons (vertices) in each layer
  # lifesign indicates how much is printed during the calculation of the NN
  # err.fct = sse = sum of squared error
  
  list_all_models[[l]][[j]][[i]]<-NeuralModel1
} else if (l == 4) { # valuable simple info on why a deeper NN: https://machinelearningmastery.com/how-to-configure-the-number-of-layers-and-nodes-in-a-neural-network/ 
  # neural network 2 layers
  train_for_NN <- cbind(train, class.ind(train$conflict_boolean)) # creates a membership matrix for the two levels of the factor variable conflict_boolean zodat we een classificatie neuralnet kunnen maken
  train_for_NN <- na.omit(train_for_NN)
  form <- as.formula(paste("peace + conflict ~", predictorVars_for_form))
  r <- (length(predictorVars)/length(levels(train$conflict_boolean)))^(1/3)
  n_hidden_nodes_1 <- round(length(levels(train$conflict_boolean))*(r^2)) # Masters' geometric pyramid rule (of thumb)
  n_hidden_nodes_2 <- round(length(levels(train$conflict_boolean))*r)
  
  NeuralModel2 <- neuralnet(formula=form, data=train_for_NN, rep=1, hidden=c(n_hidden_nodes_1,n_hidden_nodes_2), act.fct = "logistic", linear.output=FALSE, threshold = learning_threshold_2, lifesign = 'full') 
  
  list_all_models[[l]][[j]][[i]]<-NeuralModel2
  }else {warning("warning: which modelling approach are we trying?")}

} # einde loop variable sets
} # end of loop for modelling approach
} # end of loop for k-fold cross-validation

save.image('models_calculated.Rdata')

# rm(list = ls(all = TRUE)) # alles opruimen
# setwd("/Users/marie/Box/my phd documents/inhoud/modelling/2019 jan - GCRI env variables RF and NN - adaptations for paper 2/GCRI model")
# load('models_calculated.Rdata')


#===========================================================================================================================#

#### creating predictions over k folds                                                                       ####

#===========================================================================================================================#

for(i in 1:k){
  print(paste("predictions fold nr ",i))

for (crossvalidation in c("test","train")) { # loop over both test and train data
  print(paste("  predictions",crossvalidation,"set"))
  if (crossvalidation == "test") {
    temporary.data<-test.sets[[i]]
    } else temporary.data<-train.sets[[i]]


#### actual observations
#-----------------------
    
if (depVar=="conflict_boolean_NP") {
  temporary.data$actual_values <- temporary.data$conflict_boolean_NP
} else if (depVar=="conflict_boolean_SN") {
  temporary.data$actual_values <- temporary.data$conflict_boolean_SN
} else if (depVar=="conflict_boolean") {
  temporary.data$actual_values <- temporary.data$conflict_boolean
} else warning("wrong independent variable")


#### predicted values
#--------------------
    
for(l in 1:nr_of_models){
  print(paste("     predictions model approach",modelling_approaches[l]))  
    
for(j in 1:nr_of_variable_sets){ #loop loopt tot ergens in het einde van de performance berekeningem
    print(paste("         predicitons Var Set",predictorVars_set_names[j])) # to follow where we are in the calculations
    ChosenPredVars <- predictorVars_sets[[j]]
    
# filter out the models that didn't converge and set the predictions to NAs (6 of the NN did not converge)

if (l==3 & j==1 & i==4) {
  temporary.data$prediction_NN1_orig<-NA
} else if (l==3 & j==3 & i==9) {
  temporary.data$prediction_NN1_env<-NA
} else if (l==4 & j==1 & i==5) {
  temporary.data$prediction_NN2_orig<-NA
} else if (l==4 & j==1 & i==6) {
  temporary.data$prediction_NN2_orig<-NA
} else if (l==4 & j==3 & i==5) {
  temporary.data$prediction_NN2_env<-NA
} else if (l==4 & j==3 & i==6) {
  temporary.data$prediction_NN2_env<-NA
} else    
    
    
# for all other models calculate the predicitons using the trained models
    
  if (l == 1) { # linear regr
    
    if (j == 1) { # orig var set
      temporary.data$prediction_LR_orig <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'response')
    } else if (j == 2) { # socio-econ
      temporary.data$prediction_LR_socio_econ <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'response')
    } else if (j == 3) { # new env
      temporary.data$prediction_LR_env <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'response')
    } else if (j == 4) { # orig + new env
      temporary.data$prediction_LR_orig_env <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'response')
    } else warning("wrong var set")  
    
    
  } else if (l == 2) { # random forest 
    
    if (j == 1) { # orig var set
      predictions_tempRF <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'prob')
      temporary.data$prediction_RF_orig <- predictions_tempRF[,"conflict"] # beause the RF models predicts probabilities for both the peace and conflict class in a matrix with two colums
    } else if (j == 2) { # socio-econ
      predictions_tempRF <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'prob')
      temporary.data$prediction_RF_socio_econ <- predictions_tempRF[,"conflict"]
    } else if (j == 3) { # new env
      predictions_tempRF <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'prob')
      temporary.data$prediction_RF_env <- predictions_tempRF[,"conflict"]
    } else if (j == 4) { # orig + new env
      predictions_tempRF <- predict(list_all_models[[l]][[j]][[i]],newdata = temporary.data, type = 'prob')
      temporary.data$prediction_RF_orig_env <- predictions_tempRF[,"conflict"]
    } else warning("wrong var set")
  
    
  } else if (l == 3) { # neural net 1
    predictors <- subset(temporary.data, select = ChosenPredVars)
    
    if (j == 1) { # orig var set
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN1_orig <- predictions_tempNN$net.result[,2]
    } else if (j == 2) { # socio-econ
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN1_socio_econ <- predictions_tempNN$net.result[,2]
    } else if (j == 3) { # new env
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN1_env <- predictions_tempNN$net.result[,2]
    } else if (j == 4) { # orig + new env
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN1_orig_env <- predictions_tempNN$net.result[,2]
    } else warning("wrong var set")
    
    
  } else if (l == 4) { # neural net 2
    predictors <- subset(temporary.data, select = ChosenPredVars)
    
    if (j == 1) { # orig var set
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN2_orig <- predictions_tempNN$net.result[,2]
    } else if (j == 2) { # socio-econ
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN2_socio_econ <- predictions_tempNN$net.result[,2]
    } else if (j == 3) { # new env
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN2_env <- predictions_tempNN$net.result[,2]
    } else if (j == 4) { # orig + new env
      predictions_tempNN <- compute(list_all_models[[l]][[j]][[i]],predictors)
      temporary.data$prediction_NN2_orig_env <- predictions_tempNN$net.result[,2]
    } else warning("wrong var set")
    
  } else warning("wrong modelling technique")

        
  } # end of var set loop
  } # end of model approach loop

    
# save the predictions in the right test or train dataset within their list by overwriting them
    if (crossvalidation == "test") {
      test.sets[[i]]<-temporary.data
    } else train.sets[[i]]<-temporary.data    

    
# Put all obserations and predictions into one dataframe, to visualize predicted vs. observed values, residuals, etc.
temporary.data$fold_number <- i # add the fold number as variable to the dataframe
temporary.data$crossvalidation <- crossvalidation # add whether this is a test or train dataset prediction

if (!exists("predictions_for_performance_visualisations")) {predictions_for_performance_visualisations <- temporary.data} # if the output does not exist yet, create it (only first time)
else predictions_for_performance_visualisations <- rbind(predictions_for_performance_visualisations,temporary.data) # combine two dataframes by rows if the object does exist already

    
} # end of loop over test-train
} # end of loop for k-fold cross-validation

write.csv(predictions_for_performance_visualisations,file.path(getwd(),"/results/predictions_for_performance_visualisations.csv"))


#===========================================================================================================================#

#### investigating model performance                                                                        ####

#===========================================================================================================================#


# show how a neural network looks
# -------------------------------

#if (n_hidden_nodes<=12 AND l==3){
#  NeuralModel$result.matrix
#  plot(NeuralModel2)
#}

for(i in 1:k){
  print(paste("performance calculations fold nr ",i))
  
for (crossvalidation in c("test","train")) { # loop over both test and train data
  print(paste("  performance",crossvalidation,"set"))
  
  if (crossvalidation == "test") {
    temporary.data<-test.sets[[i]]
  } else temporary.data<-train.sets[[i]]  
  
for(l in 1:nr_of_models){
  print(paste("     performance model approach",modelling_approaches[l]))  
    
for(j in 1:nr_of_variable_sets){ 
  print(paste("         performance Var Set",predictorVars_set_names[j])) # to follow where we are in the calculations
  
  
# measures of fit
# ---------------
  
observed_probabilities <- ifelse(temporary.data$conflict_boolean=="peace",0,1)
column_of_concerned_prediction <- (50+(4*(l-1))+j)
predicted_probabilities <- temporary.data[,column_of_concerned_prediction]
  
conf_matr <- confusion.matrix(observed_probabilities,predicted_probabilities, threshold = 0.5)
  TN <- conf_matr[1,1]
  TP <- conf_matr[2,2]
  FN <- conf_matr[1,2]
  FP <- conf_matr[2,1]
kappa <- Kappa(conf_matr) # watch out, the function kappa needs to be with capital K otherwise it is a different function
AUC <- auc(observed_probabilities,predicted_probabilities)
observed_mean <- mean(observed_probabilities, na.rm = TRUE)
predicted_mean <- mean(predicted_probabilities, na.rm = TRUE)
Brier <- mean((observed_probabilities-predicted_probabilities)^2, na.rm = TRUE)
MAE <- mean(abs(observed_probabilities-predicted_probabilities), na.rm = TRUE)
R2_tjur <- mean(predicted_probabilities[observed_probabilities==1], na.rm = TRUE) - mean(predicted_probabilities[observed_probabilities==0], na.rm = TRUE)

accuracy <- (TP+TN)/(TP+TN+FP+FN)
sensitivity <- (TP)/(TP+FN)
specificity <- (TN)/(TN+FP)
  

# gather all performance metrics over all probability models in one data.frame
# ----------------------------------------------------------------------------

fold_number <- i
modelperformance_probability_one_model <- data.frame(fold_number, crossvalidation, modelling_approaches[l],predictorVars_set_names[j],
                                                    TN,TP,FN,FP,kappa,AUC,observed_mean,predicted_mean,Brier,MAE,R2_tjur,accuracy,sensitivity,specificity)
  
if (!exists("modelperformance_probability")) {modelperformance_probability <- modelperformance_probability_one_model} # if the output does not exist yet, create it (only first time)
else modelperformance_probability <- rbind(modelperformance_probability,modelperformance_probability_one_model) #M combine two dataframes by rows if the object does exist already
        
} # end of var set loop
} # end of model approach loop    
} # end loop over test and train for performance
} # end loop over k folds for performance

write.csv(modelperformance_probability,file.path(getwd(),"/results/model_performance_probability.csv"))


#===========================================================================================================================#

#### variable importance and interactions                                                                                ####

#===========================================================================================================================#

#### Variable importacne
#-----------------------

# select the RF model with the median AUC

median_RF_models <- list()
importance_frames <- list()

for(j in 1:nr_of_variable_sets){
  
subset_model_perf <- subset(modelperformance_probability, modelperformance_probability$modelling_approaches.l. == modelling_approaches[2] & modelperformance_probability$predictorVars_set_names.j. == predictorVars_set_names[j] & modelperformance_probability$crossvalidation == "test")
median_AUC_subset <- median(c(subset_model_perf$AUC,1)) # add one row with 100 to make the number of runs uneven so it can take the median, 1 above the half
median_fold <- subset_model_perf$fold_number[subset_model_perf$AUC==median_AUC_subset]
median_RF_models[[j]] <- list_all_models[[2]][[j]][[median_fold]]

# calculate importance frame for variable importance
importance_frames[[j]] <- measure_importance(median_RF_models[[j]], mean_sample = "all_trees")

write.csv(importance_frames[[j]],paste0(getwd(),"/results/importance_frame_",predictorVars_set_names[j],".csv"))
} # end of var set loop

#save(importance_frames, file = "results/importance_frames - all var sets.rda")
#load("results/importance_frames - all var sets.rda")

for(j in 1:nr_of_variable_sets){
plot_multi_way_importance(importance_frames[[j]], x_measure = "accuracy_decrease", y_measure = "mean_min_depth", size_measure = "no_of_nodes", no_of_labels = 10, main = predictorVars_set_names[j])
} # end of var set loop
# By default 10 top variables in the plot are highlighted in blue and labeled (no_of_labels) ??? these are selected using the function important_variables, i.e. using the sum of rankings based on importance measures used in the plot (more variables may be labeled if ties occur).


#### variable interactions: conditional minimal depth
#----------------------------------------------------

#select important variables to investigate the interactions of

interactions_frames <- list()

for(j in 1:nr_of_variable_sets){
  print(paste("importance calculations Var Set ",predictorVars_set_names[j]))

#selected_vars <- important_variables(importance_frames[[j]], k=5, measures = c("accuracy_decrease", "mean_min_depth", "no_of_nodes")) # select the 5 most important variables
selected_vars <- predictorVars_sets[[j]]  # just all variables in the set, thus calculating all possible interactions
interactions_frames[[j]] <- min_depth_interactions(median_RF_models[[j]], selected_vars, mean_sample = "all_trees")

write.csv(interactions_frames[[j]],paste0(getwd(),"/results/interactions_frame_",predictorVars_set_names[j],".csv"))
} # end of var set loop


#### calculate the average (over all trees in the forest) max and average decision path depth
#--------------------------------------------------------------------------------------------

# to compare the unconditional and conditional minimal depths with

for(j in 1:nr_of_variable_sets){
  print(paste("tree sizes Var Set ",predictorVars_set_names[j]))

leaf_depths_per_forest<-matrix(ncol=2,nrow=ntree)

for(tree_number in 1:ntree){

temporary_tree <- getTree(median_RF_models[[j]],k=tree_number, labelVar = TRUE)

leaf_nr<-0
leaf_depths_per_tree<-vector()
for (node in 1:nrow(temporary_tree)){
  if (temporary_tree$status[node] == -1){ # find the leaves
    leaf_nr<-leaf_nr+1
    depth<-1
    upper_node<-which(temporary_tree[,1:2]==node,arr.ind = TRUE)[1] #find the node above the leave
    while(upper_node != 1){ # find the nodes above the node above the leave until at the root)
      upper_node<-which(temporary_tree[,1:2]==upper_node,arr.ind = TRUE)[1]
      depth<-depth+1
    } # end of while loop over uppernodes
    leaf_depths_per_tree[leaf_nr]<-depth
  } # end of if over all leaves
} # en dof loop over all nodes
leaf_depths_per_tree

leaf_depths_per_forest[tree_number,1] <- max(leaf_depths_per_tree)
leaf_depths_per_forest[tree_number,2] <- median(leaf_depths_per_tree)
} # end of loop over all trees

print(paste("   average max tree depth of ",mean(leaf_depths_per_forest[,1])))
print(paste("   average average tree depth of ",mean(leaf_depths_per_forest[,2])))

} # end of loop over variable sets

